/*
 * This code is part of the strategy game operational-space.
 * operational-space comes with ABSOLUTELY NO WARRANTY and is licensed under GPL-2.0.
 *    Copyright (C) 2024  VegOwOtenks, Sleppo04
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef COMMON_FREELIST_H_
#define COMMON_FREELIST_H_

#include "../dynamicarray/dynamicarray.h"


// Don't put packed structs in there
// List structure optimized for leaving holes inside and filling them later

typedef struct FreeList_s {
    DynamicArray list;

    size_t first_free;
    // Index of the first free element

    size_t object_size;
} FreeList;


int FreeList_Create(FreeList* target, size_t object_size, size_t initial_capacity, allocator_t* allocator);
void FreeList_Destroy(FreeList* fl);

int FreeList_Allocate(FreeList* fl, size_t* index);

int FreeList_Free(FreeList* fl, size_t index);
int FreeList_FreeSorted(FreeList* fl, size_t index);

// Store or load `object-size` bytes
int FreeList_Load(FreeList* fl, void* destination, size_t index);
int FreeList_Store(FreeList* fl, size_t index, void* source);

// Get the pointer of the element at index
// DO NOT keep this pointer around in-between allocate() or free() calls on the FreeList
void* FreeList_GetPointer(FreeList* fl, size_t index);

#endif /* SRC_COMMON_FREELIST_FREELIST_H_ */
